% Demo script for the NMF algorithms described in this paper
%
% C. Fevotte, N. Bertin and J.-L. Durrieu. "Nonnegative matrix factorization
% with the Itakura-Saito divergence. With application to music analysis,"
% Neural Computation, vol. 21, no 3, Mar. 2009
%
% Report bugs to Cedric Fevotte
% fevotte -at- telecom-paristech.fr
% Checked 10/02/09


% Generate random nonnegative data
F = 10; 
N = 100;
V = abs(randn(F,N)) + ones(F,N);

% Generate random nonnegative initialization
K = 5;
H_ini = abs(randn(K,N)) + ones(K,N);
W_ini = abs(randn(F,K)) + ones(F,K);

%% Compute factorizations %%
n_iter = 500;

% IS-NMF/MU : multiplicative updates
[W_mu, H_mu, cost_mu] = is_nmf_mu(V, n_iter, W_ini, H_ini);

% IS-NMF/EM : SAGE updates
[W_em, H_em, cost_em] = is_nmf_em(V, n_iter, W_ini, H_ini);

% IS-NMF/IG : SAGE updates with IG Markov chain prior on H
alpha = 5 * ones(K,1);
[W_ig, H_ig, cost_ig] = is_nmf_ig(V, n_iter, W_ini, H_ini, alpha);

% Plot costs
figure;
semilogy([cost_mu' cost_em' cost_ig'])
legend('IS-NMF/MU','IS-NMF/EM','IS-NMF/IG')

% Plot W
figure;
for k=1:K
    subplot(K,1,k)
    plot([W_mu(:,k) W_em(:,k) W_ig(:,k)])
    legend('IS-NMF/MU','IS-NMF/EM','IS-NMF/IG')
    title(['w_' int2str(k)])    
end

% Plot H
figure;
title('H')
for k=1:K
    subplot(K,1,k)
    plot([H_mu(k,:)' H_em(k,:)' , H_ig(k,:)'])
    legend('IS-NMF/MU','IS-NMF/EM','IS-NMF/IG')
    title(['h_' int2str(k)])        
end